#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
	/// <summary>
    /// Calculates Exponential Volume Weighted Average Price
	/// 
	/// Code Written by Tyler Moore
	/// www.FreeIndicators.Com
	/// 
	/// For modifications to this code or to have your own custom code written
	/// please visit www.MooreTechLLC.com	
    /// </summary>
    [Description("Exponential Volume Weighted Average Price")]
    public class mtExpVWAP : Indicator
    {
        #region Variables
        // Wizard generated variables
        private int period = 14; // Default setting for Period
        // User defined variables (add any user defined variables below)
		private double alpha = 0;
		private DataSeries num, den;
        #endregion

        protected override void Initialize()
        {
            Add(new Plot(Color.FromKnownColor(KnownColor.Blue), PlotStyle.Line, "VWEMA"));
            Overlay				= true;
        }
		
		protected override void OnStartUp()
		{
			alpha = 1-2/((double)period+1);
			
			num = new DataSeries(this);
			den = new DataSeries(this);
		}

        protected override void OnBarUpdate()
        {
            if (CurrentBar==0)
			{
				num[0] = (1-alpha)*Volume[0]*Input[0];
				den[0] = (1-alpha)*Volume[0];
			}
			else
			{
				num[0] = alpha*num[1]+(1-alpha)*Volume[0]*Input[0];
				den[0] = alpha*den[1]+(1-alpha)*Volume[0];
			}
			
			if (den[0]!=0)
				VEMA[0] = num[0]/den[0];
        }

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries VEMA
        {
            get { return Values[0]; }
        }

        [Description("")]
        [GridCategory("Parameters")]
        public int Period
        {
            get { return period; }
            set { period = Math.Max(1, value); }
        }
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private mtExpVWAP[] cachemtExpVWAP = null;

        private static mtExpVWAP checkmtExpVWAP = new mtExpVWAP();

        /// <summary>
        /// Exponential Volume Weighted Average Price
        /// </summary>
        /// <returns></returns>
        public mtExpVWAP mtExpVWAP(int period)
        {
            return mtExpVWAP(Input, period);
        }

        /// <summary>
        /// Exponential Volume Weighted Average Price
        /// </summary>
        /// <returns></returns>
        public mtExpVWAP mtExpVWAP(Data.IDataSeries input, int period)
        {
            if (cachemtExpVWAP != null)
                for (int idx = 0; idx < cachemtExpVWAP.Length; idx++)
                    if (cachemtExpVWAP[idx].Period == period && cachemtExpVWAP[idx].EqualsInput(input))
                        return cachemtExpVWAP[idx];

            lock (checkmtExpVWAP)
            {
                checkmtExpVWAP.Period = period;
                period = checkmtExpVWAP.Period;

                if (cachemtExpVWAP != null)
                    for (int idx = 0; idx < cachemtExpVWAP.Length; idx++)
                        if (cachemtExpVWAP[idx].Period == period && cachemtExpVWAP[idx].EqualsInput(input))
                            return cachemtExpVWAP[idx];

                mtExpVWAP indicator = new mtExpVWAP();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.Period = period;
                Indicators.Add(indicator);
                indicator.SetUp();

                mtExpVWAP[] tmp = new mtExpVWAP[cachemtExpVWAP == null ? 1 : cachemtExpVWAP.Length + 1];
                if (cachemtExpVWAP != null)
                    cachemtExpVWAP.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cachemtExpVWAP = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Exponential Volume Weighted Average Price
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.mtExpVWAP mtExpVWAP(int period)
        {
            return _indicator.mtExpVWAP(Input, period);
        }

        /// <summary>
        /// Exponential Volume Weighted Average Price
        /// </summary>
        /// <returns></returns>
        public Indicator.mtExpVWAP mtExpVWAP(Data.IDataSeries input, int period)
        {
            return _indicator.mtExpVWAP(input, period);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Exponential Volume Weighted Average Price
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.mtExpVWAP mtExpVWAP(int period)
        {
            return _indicator.mtExpVWAP(Input, period);
        }

        /// <summary>
        /// Exponential Volume Weighted Average Price
        /// </summary>
        /// <returns></returns>
        public Indicator.mtExpVWAP mtExpVWAP(Data.IDataSeries input, int period)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.mtExpVWAP(input, period);
        }
    }
}
#endregion
